## 字符串常用方法

- C语言中供了丰富的字符串处理函数,大致可分为字符串的输入、输出、合并、修改、比较、转 换、复制、搜索几类。 

+ 使用这些函数可大大减轻编程的负担。
+ 使用输入输出的字符串函数,在使用前应包含头文件"stdio.h"
+ 使用其它字符串函数则应包含头文件"string.h"

---

- 字符串输出函数:puts
- 格式: puts(字符数组名)
- 功能:把字符数组中的字符串输出到显示器。即在屏幕上显示该字符串。
- 优点:

+ 自动换行
+ 可以是数组的任意元素地址

- 缺点

+ 不能自定义输出格式, 例如 puts("hello %i");

```c
char ch[] = "lnj";
puts(ch); //输出结果: lnj
```

>+ puts函数完全可以由printf函数取代。当需要按一定格式输出时,通常使用printf函数

---

- 字符串输入函数:gets
- 格式: gets (字符数组名)
- 功能:从标准输入设备键盘上输入一个字符串。

```c
char ch[30];
gets(ch); // 输入:lnj
puts(ch); // 输出:lnj
```

>+  可以看出当输入的字符串中含有空格时,输出仍为全部字符串。说明gets函数并不以空格作为字符串输入结束的标志,而只以回车作为输入结束。这是与scanf函数不同的。
>+  注意gets很容易导致数组下标越界，是一个不安全的字符串操作函数

---

- 字符串长度
- 利用sizeof字符串长度

+ 因为字符串在内存中是逐个字符存储的,一个字符占用一个字节,所以字符串的结束符长度也是占用的内存单元的字节数。

```c
    char name[] = "it666";
    int size = sizeof(name);// 包含\0
    printf("size = %d\n", size); //输出结果:6
```

---

- 利用系统函数

+ 格式: strlen(字符数组名)
+ 功能:测字符串的实际长度(不含字符串结束标志‘\0’)并作为函数返回值。

```c
    char name[] = "it666";
    size_t len = strlen(name2);
    printf("len = %lu\n", len); //输出结果:5
```

---

- 以“\0”为字符串结束条件进行统计

```c
/**
 *  自定义方法计算字符串的长度
 *  @param name 需要计算的字符串
 *  @return 不包含\0的长度
 */
int myStrlen2(char str[])
{
    //    1.定义变量保存字符串的长度
    int length = 0;
    while (str[length] != '\0')
    {
        length++;//1 2 3 4
    }
    return length;
}
/**
 *  自定义方法计算字符串的长度
 *  @param name  需要计算的字符串
 *  @param count 字符串的总长度
 *  @return 不包含\0的长度
 */
int myStrlen(char str[], int count)
{
//    1.定义变量保存字符串的长度
    int length = 0;
//    2.通过遍历取出字符串中的所有字符逐个比较
    for (int i = 0; i < count; i++) {
//        3.判断是否是字符串结尾
        if (str[i] == '\0') {
            return length;
        }
        length++;
    }
    return length;
}
```

---

- 字符串连接函数:strcat
- 格式: strcat(字符数组名1,字符数组名2)
- 功能:把字符数组2中的字符串连接到字符数组1 中字符串的后面,并删去字符串1后的串标志 “\0”。本函数返回值是字符数组1的首地址。

```c
char oldStr[100] = "welcome to";
char newStr[20] = " lnj";
strcat(oldStr, newStr);
puts(oldStr); //输出: welcome to lnj"
```

>+ 本程序把初始化赋值的字符数组与动态赋值的字符串连接起来。要注意的是,字符数组1应定义足 够的长度,否则不能全部装入被连接的字符串。

---

- 字符串拷贝函数:strcpy
  `- 格式: strcpy(字符数组名1,字符数组名2)
  `- 功能:把字符数组2中的字符串拷贝到字符数组1中。串结束标志“\0”也一同拷贝。字符数名2, 也可以是一个字符串常量。这时相当于把一个字符串赋予一个字符数组。

```c
char oldStr[100] = "welcome to";
char newStr[50] = " lnj";
strcpy(oldStr, newStr);
puts(oldStr); // 输出结果:  lnj // 原有数据会被覆盖
```

>+ 本函数要求字符数组1应有足够的长度,否则不能全部装入所拷贝的字符串。

---

- 字符串比较函数:strcmp
- 格式: strcmp(字符数组名1,字符数组名2) 
- 功能:按照ASCII码顺序比较两个数组中的字符串,并由函数返回值返回比较结果。

+ 字符串1=字符串2,返回值=0;
+ 字符串1>字符串2,返回值>0;
+ 字符串1<字符串2,返回值<0。

```c
    char oldStr[100] = "0";
    char newStr[50] = "1";
    printf("%d", strcmp(oldStr, newStr)); //输出结果:-1
    char oldStr[100] = "1";
    char newStr[50] = "1";
    printf("%d", strcmp(oldStr, newStr));  //输出结果:0
    char oldStr[100] = "1";
    char newStr[50] = "0";
    printf("%d", strcmp(oldStr, newStr)); //输出结果:1
```

## 

最后，如果有任何疑问，请加微信 **leader_fengy** 拉你进学习交流群。

开源不易，码字不易，如果觉得有价值，欢迎分享支持。