## auto和register关键字

- auto关键字(忘记)

+ 只能修饰局部变量, 局部变量如果没有其它修饰符, 默认就是auto的
+ 特点: 随用随开, 用完即销

```c
auto int num; // 等价于 int num;
```

- register关键字(忘记)

+  只能修饰局部变量, 原则上将内存中变量提升到CPU寄存器中存储, 这样访问速度会更快
+  但是由于CPU寄存器数量相当有限,  通常不同平台和编译器在优化阶段会自动转换为auto

```c
register int num; 
```

## static关键字

- **对局部变量的作用**

+ 延长局部变量的生命周期,从程序启动到程序退出,但是它并没有改变变量的作用域
+ 定义变量的代码在整个程序运行期间仅仅会执行一次

```c
#include <stdio.h>
void test();
int main()
{
    test();
    test();
    test();

    return 0;
}
void test(){
    static int num = 0; // 局部变量
    num++; 
    // 如果不加static输出 1 1 1
    // 如果添加static输出 1 2 3
    printf("num = %i\n", num); 
}
```

- **对全局变量的作用**
- 全局变量分类：

+ 内部变量:只能在本文件中访问的变量
+ 外部变量:可以在其他文件中访问的变量,默认所有全局变量都是外部变量

- 默认情况下多个同名的全局变量共享一块空间, 这样会导致全局变量污染问题
- 如果想让某个全局变量只在某个文件中使用, 并且不和其他文件中同名全局变量共享同一块存储空间, 那么就可以使用static

```c
// A文件中的代码
int num; // 和B文件中的num共享
void test(){
    printf("ds.c中的 num = %i\n", num);
}
```

```c
// B文件中的代码
#include <stdio.h>
#include "ds.h"

int num; // 和A文件中的num共享
int main()
{
    num = 666;
    test(); // test中输出666
    return 0;
}
```

```c
// A文件中的代码
static int num; // 不和B文件中的num共享
void test(){
    printf("ds.c中的 num = %i\n", num);
}
```

```c
// B文件中的代码
#include <stdio.h>
#include "ds.h"

int num; // 不和A文件中的num共享
int main()
{
    num = 666;
    test(); // test中输出0
    return 0;
}
```

---

## extern关键字

- 对局部变量的作用

+ extern不能用于局部变量
+ extern代表声明一个变量, 而不是定义一个变量, 变量只有定义才会开辟存储空间
+ 所以如果是局部变量, 虽然提前声明有某个局部变量, 但是局部变量只有执行到才会分配存储空间

```c
#include <stdio.h>

int main()
{
    extern int num;
    num = 998; // 使用时并没有存储空间可用, 所以声明了也没用
    int num; // 这里才会开辟
    printf("num = %i\n", num);
    return 0;
}
```

- 对全局变量的作用

+ 声明一个全局变量, 代表告诉编译器我在其它地方定义了这个变量, 你可以放心使用

```c
#include <stdio.h>

int main()
{
    extern int num; // 声明我们有名称叫做num变量
    num = 998; // 使用时已经有对应的存储空间
    printf("num = %i\n", num);
    return 0;
}
int num; // 全局变量, 程序启动就会分配存储空间
```

## 

最后，如果有任何疑问，请加微信 **leader_fengy** 拉你进学习交流群。

开源不易，码字不易，如果觉得有价值，欢迎分享支持。